from typing import Any, Dict, List

import numpy as np

from olympict.files.o_file import OlympFile
from olympict.files.o_image import OlympImage
from olympict.types import Img, Size


class OlympBatch(OlympFile):
    def __init__(self, data: Img, metadata: List[Dict[str, Any]] = []):
        assert len(data.shape) >= 4
        self.data = data
        self.metadata = metadata

    @property
    def size(self) -> Size:
        _, h, w, _ = self.data.shape
        return (w, h)

    @staticmethod
    def from_images(images: List[OlympImage]) -> "OlympBatch":
        data = np.array([i.img for i in images])
        metadata = [i.metadata for i in images]

        return OlympBatch(data, metadata)



import os
import shutil
from typing import Optional

from olympict.files.o_file import OlympFile


class OlympVid(OlympFile):
    __id = 0

    def __init__(self, path: Optional[str] = None):
        super().__init__(path)
        if path is None:
            self._tmp_path = f"./{self.__id}.mp4"
            self.__id += 1
        self._tmp_path = ""
        self._fps = 25

    def change_folder_path(self, new_folder_path: str):
        old_path: str = self.path
        self.path = os.path.join(new_folder_path, os.path.basename(self.path))
        shutil.move(old_path, self.path)

    def get_temp_path(self) -> str:
        return self._tmp_path



from typing import Any, List, Tuple

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal


PointRelative = Tuple[float, float]
"""PointRelative
is a tuple composed of:
x: [0-1] float
y: [0-1] float
"""

PointAbsolute = Tuple[int, int]
"""PointAbsolute
is a tuple composed of:
x: [0-width] int
y: [0-height] int
"""

BBoxRelative = Tuple[float, float, float, float, str, float]
"""BBoxRelative
is a tuple composed of:
x1: [0-1] float
y1: [0-1] float
x2: [0-1] float
y2: [0-1] float
class: str
confidence: [0-1] float
"""

BBoxAbsolute = Tuple[int, int, int, int, str, float]
"""BBoxAbsolute
is a tuple composed of:
x1: [0-width] int
y1: [0-height] int
x2: [0-width] int
y2: [0-height] int
class: str
confidence: [0-1] float
"""

PolygonRelative = List[PointRelative]
"""PolygonRelative
A list of PointsRelative to draw a filled shape
"""


PolygonAbsolute = List[PointAbsolute]
"""PolygonAbsolute
A list of PointsAbsolute to draw a filled shape
"""


LineRelative = List[PointRelative]
"""LineRelative
A list of PointsRelative to be joined by a segment
"""

LineAbsolute = List[PointAbsolute]
"""LineAbsolute
A list of PointsAbsolute to be joined by a segment
"""


ImgFormat = Literal["png", "jpeg", "bmp"]
VidFormat = Literal["mp4", "mkv", "avi"]

Size = Tuple[int, int]
"""Size
is a tuple dedicated to image sizing:
w: int image width in pixels
h: int image height in pixels
"""

Color = Tuple[int, int, int]
"""Color
is a tuple dedicated to color representation using BGR format
b: [0-255] int blue channel
g: [0-255] int green channel
r: [0-255] int red channel
"""


Img = Any
"""Img
represents the image format, usually a numpy uint8 array or a cv2.Mat
"""
